/*!\file TriaVertexElementResult.c
 * \brief: implementation of the TriaVertexElementResult object
 */

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "./ElementResultLocal.h"
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../DataSet/DataSet.h"
#include "../../include/include.h"

/*Object constructors and destructor*/
/*FUNCTION TriaVertexElementResult::TriaVertexElementResult(){{{1*/
TriaVertexElementResult::TriaVertexElementResult(){
	return;
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::TriaVertexElementResult(int in_enum_type,double* in_values,int in_step, double in_time){{{1*/
TriaVertexElementResult::TriaVertexElementResult(int in_enum_type,double* in_values,int in_step, double in_time):TriaVertexInput(in_enum_type,in_values){

	step=in_step;
	time=in_time;
	printf("file: TriaVertexElementResult.cpp line: %i\n",__LINE__); 
	this->Echo();
	printf("file: TriaVertexElementResult.cpp line: %i\n",__LINE__); 
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::~TriaVertexElementResult(){{{1*/
TriaVertexElementResult::~TriaVertexElementResult(){
	return;
}
/*}}}*/

/*Object management*/
/*FUNCTION TriaVertexElementResult::copy{{{1*/
Object* TriaVertexElementResult::copy() {
	
	return new TriaVertexElementResult(this->enum_type,this->values,this->step,this->time);

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::DeepEcho{{{1*/
void TriaVertexElementResult::DeepEcho(void){
		
	printf("TriaVertexElementResult:\n");
	printf("   step: %i\n",this->step);
	printf("   time: %g\n",this->time);
	this->TriaVertexInput::DeepEcho();

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::Demarshall{{{1*/
void  TriaVertexElementResult::Demarshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   i;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*this time, no need to get enum type, the pointer directly points to the beginning of the 
	 *object data (thanks to DataSet::Demarshall):*/
	memcpy(&enum_type,marshalled_dataset,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(&values,marshalled_dataset,sizeof(values));marshalled_dataset+=sizeof(values);
	memcpy(&time,marshalled_dataset,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(&step,marshalled_dataset,sizeof(step));marshalled_dataset+=sizeof(step);

	/*return: */
	*pmarshalled_dataset=marshalled_dataset;
	return;
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::Echo {{{1*/
void TriaVertexElementResult::Echo(void){
	this->DeepEcho();
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::Enum{{{1*/
int TriaVertexElementResult::Enum(void){

	return TriaVertexElementResultEnum;

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::EnumType{{{1*/
int TriaVertexElementResult::EnumType(void){

	return this->enum_type;

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::Id{{{1*/
int    TriaVertexElementResult::Id(void){ return -1; }
/*}}}*/
/*FUNCTION TriaVertexElementResult::Marshall{{{1*/
void  TriaVertexElementResult::Marshall(char** pmarshalled_dataset){

	char* marshalled_dataset=NULL;
	int   enum_value=0;

	/*recover marshalled_dataset: */
	marshalled_dataset=*pmarshalled_dataset;

	/*get enum value of TriaVertexElementResult: */
	enum_value=TriaVertexElementResultEnum;
	
	/*marshall enum: */
	memcpy(marshalled_dataset,&enum_value,sizeof(enum_value));marshalled_dataset+=sizeof(enum_value);
	
	/*marshall TriaVertexElementResult data: */
	memcpy(marshalled_dataset,&enum_type,sizeof(enum_type));marshalled_dataset+=sizeof(enum_type);
	memcpy(marshalled_dataset,&values,sizeof(values));marshalled_dataset+=sizeof(values);
	memcpy(marshalled_dataset,&time,sizeof(time));marshalled_dataset+=sizeof(time);
	memcpy(marshalled_dataset,&step,sizeof(step));marshalled_dataset+=sizeof(step);

	*pmarshalled_dataset=marshalled_dataset;
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::MarshallSize{{{1*/
int   TriaVertexElementResult::MarshallSize(){
	
	return sizeof(values)+
		+sizeof(enum_type)
		+sizeof(time)
		+sizeof(step)
		+sizeof(int); //sizeof(int) for enum value
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::MyRank{{{1*/
int    TriaVertexElementResult::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/

/*ElementResult functions*/
/*FUNCTION TriaVertexElementResult::SpawnSingElementResult{{{1*/
ElementResult* TriaVertexElementResult::SpawnSingElementResult(int index){

	/*output*/
	SingVertexElementResult* outresult=NULL;

	/*Create new Sing result (copy of current result)*/
	ISSMASSERT(index<3 && index>=0);
	outresult=new SingVertexElementResult(this->enum_type,this->values[index],this->step,this->time);

	/*Assign output*/
	return outresult;

}
/*}}}*/
/*FUNCTION BeamVertexElementResult::SpawnBeamElementResult{{{1*/
ElementResult* TriaVertexElementResult::SpawnBeamElementResult(int* indices){

	/*output*/
	BeamVertexElementResult* outresult=NULL;
	double newvalues[2];

	/*Loop over the new indices*/
	for(int i=0;i<2;i++){

		/*Check index value*/
		ISSMASSERT(indices[i]>=0 && indices[i]<3);

		/*Assign value to new result*/
		newvalues[i]=this->values[indices[i]];
	}

	/*Create new Beam result*/
	outresult=new BeamVertexElementResult(this->enum_type,&newvalues[0],this->step,this->time);

	/*Assign output*/
	return outresult;

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::SpawnTriaElementResult{{{1*/
ElementResult* TriaVertexElementResult::SpawnTriaElementResult(int* indices){

	/*output*/
	TriaVertexElementResult* outresult=NULL;

	/*Create new Tria result (copy of current result)*/
	outresult=new TriaVertexElementResult(this->enum_type,&this->values[0],this->step,this->time);

	/*Assign output*/
	return outresult;

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::ProcessUnits(Parameters* parameters){{{1*/
void TriaVertexElementResult::ProcessUnits(Parameters* parameters){
	
	NodalValuesUnitConversion(this->values,3,this->enum_type,parameters);

}
/*}}}*/
/*FUNCTION TriaVertexElementResult::NumberOfNodalValues(void){{{1*/
int TriaVertexElementResult::NumberOfNodalValues(void){
	return 3;
}
/*}}}*/
/*FUNCTION TriaVertexElementResult::TriaVertexElementResult::PatchFill(int row, Patch* patch){{{1*/
void TriaVertexElementResult::PatchFill(int row, Patch* patch){
	
	 /*Here, we fill the result information into the patch object. First, let's remember what is in a row 
	  * of the patch object: enum_type step time element_id interpolation vertices_ids nodal_values
	  * Here, we will supply the enum_type, step, time, interpolation and nodal_values: */
	patch->fillresultinfo(row,this->enum_type,this->step,this->time,P1Enum,this->values,3);

}
/*}}}*/
